# pythran export compute_increments_dim1(float64[:, :], int)

def compute_increments_dim1(var, irx):
    'Compute the increments of var over the dim 1.'
    n1 = var.shape[1]
    n1new = (n1 - irx)
    inc_var = (var[:, irx:n1] - var[:, 0:n1new])
    return inc_var




import os
import glob
import pytest
import importlib
import numpy as np
from pythran import compile_pythrancode


def make_benchmark(benchmark_path):
    modname, _ = os.path.splitext(os.path.basename(benchmark_path))
    def the_benchmark(benchmark):
        def setup():

            with open(benchmark_path) as fd:
                code = fd.readlines()
                for line in code:
                    if line.startswith('#bench '):
                        _, bench = line.split(' ', 1)
                        tbench = bench.rsplit(';', 1)

                        setup_env = tbench[0] if len(tbench) == 2 else ""
                        run = tbench[-1].strip()
                        fun, _ = run.split('(', 1)
                        assert run
                        break
                else:
                    raise RuntimeError(f"#bench line not found in {benchmark_path}")
                env = {"builtins": __import__("builtins")}
                exec(setup_env, env)
                benchmark_run = lambda: eval(run, env)

                compiled = compile_pythrancode(modname, "".join(code))
                loader = importlib.machinery.ExtensionFileLoader(modname, compiled)
                spec = importlib.machinery.ModuleSpec(name=modname, loader=loader,
                                                      origin=compiled)
                pymod = importlib._bootstrap._load(spec)
                env[fun] = getattr(pymod, fun)
                return (compiled, benchmark_run), {}

        def target(path, runner):
            return runner()

        def teardown(path, *args):
            # Cleanup code that shouldn't be measured
            os.remove(path)

        result = benchmark.pedantic(
            target,
            setup=setup,
            teardown=teardown,
            rounds=7,          # Number of rounds to run
            warmup_rounds=1    # Number of warmup rounds
        )
    return modname, the_benchmark

for file_py in glob.glob(os.path.join(os.path.dirname(__file__), "..", "tests", "cases", "*.py")):
    with open(file_py) as fd:
        code = fd.read()
        if "#bench " not in code:
            continue
        if "#pythran export " not in code:
            continue

    modname, benchmark = make_benchmark(file_py)
    globals()[f'test_{modname}'] = benchmark
#
#
#test_allpairs = make_benchmark(os.path.join(os.path.dirname(__file__),
#                                            "cases", "allpairs.py"),
#                               )
#
#test_brownian = make_benchmark(os.path.join(os.path.dirname(__file__),
#                                            "cases", "brownian.py"),
#                               )



"""
UseOMP detects if a function use OpenMP
"""

from pythran.passmanager import FunctionAnalysis


class UseOMP(FunctionAnalysis):
    """Detects if a function use openMP"""
    ResultType = bool

    def visit_OMPDirective(self, _):
        self.result = True



import numpy
import pytest
import sys
import unittest

from pythran.tests import TestEnv
from pythran.typing import *

class TestBase(TestEnv):
    def test_simple_ann_stmt(self):
        self.run_test("def simple_ann_stmt(a):\n b: int = a; return b, b",
                      1,
                      simple_ann_stmt=[int])

    def test_dead_ann_stmt(self):
        self.run_test("def dead_ann_stmt(a): b: float = 2; return a",
                      1,
                      dead_ann_stmt=[int])

    def test_fwd_ann_stmt(self):
        self.run_test("def fwd_ann_stmt(a): b: float; b = 1.; return a + b",
                      1,
                      fwd_ann_stmt=[int])

class TestTypeAnnotation(TestEnv):

    def test_int_ann(self):
        self.run_test("def int_ann(a):\n b: int = 1; return a + b",
                      1,
                      int_ann=[int])

    def test_bool_ann(self):
        self.run_test("def bool_ann(a):\n b: bool = a; return b",
                      True,
                      bool_ann=[bool])

    def test_float_ann(self):
        self.run_test("def float_ann(a):\n b: float = 2.; return b * a",
                      1,
                      float_ann=[int])

    def test_complex_ann(self):
        self.run_test("def complex_ann(a):\n b: complex = 2.j; return b * a",
                      1,
                      complex_ann=[int])

    def test_tuple_ann(self):
        self.run_test("def tuple_ann(a):\n b: tuple[int, float] = (a, 1.); return b",
                      1,
                      tuple_ann=[int])

    def test_list_ann(self):
        self.run_test("def list_ann(a):\n b: list[int] = []; return b * a",
                      1,
                      list_ann=[int])

    def test_set_ann(self):
        self.run_test("def set_ann(a):\n b: set[int] = set(); return b",
                      1,
                      set_ann=[int])

    def test_dict_ann(self):
        self.run_test("def dict_ann(a):\n b: dict[int, str] = {}; return b",
                      1,
                      dict_ann=[int])

    def test_typeof_ann_arg(self):
        self.run_test("def typeof_ann_arg(a):\n b: dict[int, type(a)] = {}; return b",
                      1,
                      typeof_ann_arg=[int])

    def test_typeof_ann_local(self):
        self.run_test("def typeof_ann_local(a):\n c = a + 1\n b: list[type(c)] = []; return b * a, c",
                      1,
                      typeof_ann_local=[int])

    def test_ann_union(self):
        self.run_test("def typeof_ann_union(a):\n b: list[type(a) | float] = [1.]; b.append(a); return b",
                      1,
                      typeof_ann_union=[int])

    def test_ann_union_redundant(self):
        self.run_test("def typeof_ann_union_redundant(a):\n b: list[float | float] = [1.]; b.append(a); return b",
                      1,
                      typeof_ann_union_redundant=[int])

    def test_ann_union_none(self):
        self.run_test("def typeof_ann_union_none(a):\n b: list[None | int] = [a]; return b[0] is None",
                      1,
                      typeof_ann_union_none=[int])

    def test_ann_union_none_init(self):
        self.run_test("def typeof_ann_union_none_init(a):\n b: type(a) | None = None\n if a: b = a\n return b",
                      1,
                      typeof_ann_union_none_init=[int])

    def test_typeof_ann_expr(self):
        self.run_test("def typeof_ann_expr(a):\n b: list[type(a * 2)] = []; return b * a",
                      1,
                      typeof_ann_expr=[int])

    def test_typeof_ann_backward(self):
        self.run_test("""
import numpy as np
def poo(b, a):
    b.append(a)
    b.append(2)

def typeof_ann_backward(a):
    b = []
    c : type(b) = []
    poo(b, a)
    return b, c""",
                      1,
                      typeof_ann_backward=[int])

    def test_np_int_ann(self):
        self.run_test("import numpy as np\ndef npint_ann(a):\n b: np.int8 = 1; return a + b",
                      1,
                      npint_ann=[int])

    def test_np_ndarray_ann(self):
        self.run_test("import numpy as np\ndef npndarray_ann(a):\n b: np.ndarray[np.float32, 2] = np.ones((400,4), dtype=np.float32); return a + b",
                      1,
                      npndarray_ann=[int])




""" GlobalDeclarations gathers top-level declarations. """

import gast as ast
from pythran.passmanager import ModuleAnalysis

class GlobalDeclarations(ModuleAnalysis):

    """ Gather all kind of identifier defined at global scope.

    >>> import gast as ast
    >>> from pythran import passmanager
    >>> from pythran.analyses import GlobalDeclarations
    >>> node = ast.parse('''
    ... import math
    ... import math as maths
    ... from math import cos
    ... c = 12
    ... def foo(a):
    ...     b = a + 1''')
    >>> pm = passmanager.PassManager("test")
    >>> sorted(pm.gather(GlobalDeclarations, node).keys())
    ['c', 'cos', 'foo', 'math', 'maths']

    """

    ResultType = dict

    def visit_FunctionDef(self, node):
        """ Import module define a new variable name. """
        self.result[node.name] = node

    def visit_Import(self, node):
        for alias in node.names:
            self.result[alias.asname or alias.name] = alias

    def visit_ImportFrom(self, node):
        for alias in node.names:
            self.result[alias.asname or alias.name] = alias

    def visit_Name(self, node):
        if isinstance(node.ctx, ast.Store):
            self.result[node.id] = node


class NonlocalDeclarations(ModuleAnalysis):

    """ Transitively gather nonlocal declarations, per function

    >>> import gast as ast
    >>> from pythran import passmanager
    >>> from pythran.analyses import NonlocalDeclarations
    >>> node = ast.parse('''
    ... def foo(a):
    ...   def t():
    ...     def bar():
    ...       nonlocal a
    ...       a = 1
    ...     bar()
    ...   t()''')
    >>> pm = passmanager.PassManager("test")
    >>> [(n.name, l) for n, l in pm.gather(NonlocalDeclarations, node).items()]
    [('foo', set()), ('t', {'a'}), ('bar', {'a'})]

    """

    ResultType = dict

    def __init__(self):
        super().__init__()
        self.context = []
        self.locals = []
        self.nested = [set()]

    def visit_FunctionDef(self, node):
        self.nested[-1].add(node)

        self.nested.append(set())
        self.locals.append(set())

        self.context.append(node)

        self.result[node] = set()
        self.generic_visit(node)

        self.context.pop()

        locals_ = self.locals.pop()
        nested = self.nested.pop()

        for f in nested:
            self.result[node].update(self.result[f] - locals_)

    def visit_Name(self, node):
        if isinstance(node.ctx, (ast.Store, ast.Param)):
            self.locals[-1].add(node.id)

    def visit_Nonlocal(self, node):
        self.result[self.context[-1]].update(node.names)



import numpy as np
  
#pythran export declare_reduction_aggregated_type()
def declare_reduction_aggregated_type():
    return run(1000)

#omp declare reduction(+ : float[], int[] : omp_out += omp_in) initializer(omp_priv(np.empty_like(omp_orig)))

def run(n_bins):

    """
    Example of using openmp reduction to accelerate
    for loop
    """

    arr = np.zeros(n_bins)

    #omp parallel for reduction(+:arr)
    for i in range(10):
        for j in range(10):

            ij = i * 10 + j
            arr[int(ij % n_bins)] += 1

    return arr



#encoding: utf8
from pythran.tests import TestEnv

from unittest import skip, skipIf
import numpy

from pythran.typing import *
from packaging.version import Version

class TestNestedFunctions(TestEnv):

    def test_capture_argument(self):
        code = '''
def capture_argument(n):
    def foo(y):
        return n + y
    return foo(n)'''
        self.run_test(code, 42, capture_argument=[int])

    def test_capture_and_update_argument(self):
        code = '''
def capture_and_update_argument(n, i):
    def foo(y):
        n.append(y)
    foo(i)
    return n'''
        self.run_test(code, [3], 42, capture_and_update_argument=[List[int], int])

    def test_capture_argument_then_use(self):
        code = '''
def capture_argument_then_use(n):
    def foo(y):
        return n + y
    t = foo(n)
    return t, n'''
        self.run_test(code, 42, capture_argument_then_use=[int])

    def test_capture_argument_then_use_twice(self):
        code = '''
def capture_argument_then_use_twice(n):
    def foo(y):
        n.append(y)
    foo(len(n))
    foo(len(n))
    return n'''
        self.run_test(code, [42], capture_argument_then_use_twice=[List[int]])

    def test_capture_argument_then_update(self):
        code = '''
def capture_argument_then_update(n):
    def foo(y):
        return n + y
    t = foo(n)
    n += 1
    return t, n'''
        self.run_test(code, 42, capture_argument_then_update=[int])

    def test_capture_local(self):
        code = '''
def capture_local(y):
    n = y
    def foo(y):
        return n + y
    return foo(n)'''
        self.run_test(code, 42, capture_local=[int])

    def test_capture_and_update_local(self):
        code = '''
def capture_and_update_local(y, i):
    n = y
    def foo(y):
        n.append(y)
    foo(i)
    return n'''
        self.run_test(code, [3], 42, capture_and_update_local=[List[int], int])

    def test_capture_local_then_rebind(self):
        code = '''
def capture_local_then_rebind(n):
    m = []
    def foo(y):
        m.append(y)
    foo(len(m))
    p = list(m)
    m = []
    foo(len(m))
    return p, m'''
        self.run_test(code, 42, capture_local_then_rebind=[int])

    def test_redef_arg(self):
        code = '''
def redef_arg(x):
    z = 1
    def foo(x, y):
        def bar(x, y, z):
            return x, y, z
        return bar(x, y, z)
    return foo(x, "e")'''
        self.run_test(code, 42, redef_arg=[int])

    def test_ultra_nested_functions(self):
        code = '''
def ultra_nested_function(n):
    def foo(y):
        def bar(t): return t
        return bar(y)
    return foo(n)'''
        self.run_test(code, 42, ultra_nested_function=[int])

    def test_nested_with_conflicting_argument_names(self):
        code = '''
def nested_with_conflicting_argument_names(a1):
    def f2():
        r2 = a1
        def f3():
            r3 = a1
            return r3
        return r2 + f3()
    return f2()'''
        self.run_test(code, 5, nested_with_conflicting_argument_names=[int])

    def test_nonlocal_simple(self):
        code = '''
    def nonlocal_simple(x):
      def myfunc2():
        nonlocal x
        x = "hello"
      myfunc2()
      return x'''
        self.run_test(code, 'yo', nonlocal_simple=[str])

    def test_nonlocal_base(self):
        code = '''
def nonlocal_base(a):
    ret = 0
    def modify_ret (delta):
        nonlocal ret
        ret += delta
    for i in range(a):
        modify_ret (10)
    return ret'''
        self.run_test(code, 5, nonlocal_base=[int])

    def test_nonlocal_deep(self):
        code = '''
def nonlocal_deep(a):
    ret = 0
    def foo (delta):
        def bar(beta):
            nonlocal ret
            ret += delta + beta
        for i in range(a):
            bar (10)
    for i in range(a):
        foo (3)
    return ret'''
        self.run_test(code, 5, nonlocal_deep=[int])

    def test_nonlocal_shadow_global(self):
        code = '''
            x = 0
            def nonlocal_shadow_global(x):
                x = 1
                def inner():
                    nonlocal x
                    x = 2

                inner()'''
        self.run_test(code, 8, nonlocal_shadow_global=[int])

    def test_long_capture_list(self):
        code = '''
import numpy
def long_capture_list():
    arr = numpy.ones(shape = (3, 20))

    a1 = 1
    a2 = 2
    a3 = 3
    a4 = 4
    a5 = 5
    a6 = 6
    a7 = 7
    a8 = 8
    a9 = 9
    a10 = 10

    a11 = 11
    a12 = 12
    a13 = 13
    a14 = 14
    a15 = 15
    a16 = 16
    a17 = 17
    a18 = 18
    a19 = 19
    a20 = 20

    def save_step():
        arr[0][:] = [
            a1,
            a2,
            a3,
            a4,
            a5,
            a6,
            a7,
            a8,
            a9,
            a10,
            a11,
            a12,
            a13,
            a14,
            a15,
            a16,
            a17,
            a18,
            a19,
            a20,
        ]
    save_step()
    return arr'''
        self.run_test(code, long_capture_list=[])

    def test_capture_augassign(self):
        code = '''
def capture_augassign(cache, inputs):
    aaa_init = inputs[0][0][0]
    aaa = 0.
    ny = 0
    aaas = []
    def save_step():
        cache[0][ny][0][0][0][0] = aaa
    for ny in range(2):
        aaa = cache[0][ny - 1][0][0][0][0]
        aaas.append(aaa)
        aaa *= 0.9
        save_step()
    return aaas'''
        self.run_test(code, 
                      numpy.full(shape = (1, 10, 1, 1, 1, 1), fill_value =
                      123.),
                      ((numpy.asarray([20., 20.]),),),
                      capture_augassign=[NDArray[float,:,:,:,:,:,:],
                      Tuple[Tuple[NDArray[float, :]]]])



from tempfile import mkstemp
from pythran.tests import TestEnv

import unittest

from pythran.typing import List


class TestFile(TestEnv):

    def __init__(self, *args, **kwargs):
        super(TestFile, self).__init__(*args, **kwargs)
        self.file_content = """azerty\nqwerty\n\n"""

    def tempfile(self):
        filename=mkstemp()[1]
        with open(filename,"w") as f:
            f.write(self.file_content)
        self.filename = filename
        return filename

    def reinit_file(self):
        with open(self.filename,"w") as f:
            f.write(self.file_content)
        return self.filename

    def test_filename_only_constructor(self):
        filename=mkstemp()[1]
        self.run_test("""
            def filename_only_constructor(filename):
                open(filename).close()""",
            filename, filename_only_constructor=[str])

    def test_open_write(self):
        filename=mkstemp()[1]
        self.run_test("""
            def _open_write(filename):
                f = open(filename,"w+")
                f.write("azert")
                f.close()""",
            filename, _open_write=[str])
        with open(filename) as fd:
            self.assertEqual(fd.read(), "azert")

    def test_open_append(self):
        filename=mkstemp()[1]
        self.run_test("""
            def _open_append(filename):
                f = open(filename,"a")
                f.write("azert")
                f.close()""",
            filename,
            _open_append=[str])
        with open(filename) as fd:
            # * 2 because it's opened by python then pythran
            self.assertEqual(fd.read(), "azert"*2)

    def test_writing_mode_constructor(self):
        # Expecting file to be erased.
        # But python execution of test will erase it before pythran can :s
        self.tempfile()
        self.run_test("""
            def writing_mode_constructor(filename):
                f = open(filename, "w")
                f.close()""",
            self.filename,prelude=self.reinit_file, writing_mode_constructor=[str])

        with open(self.filename) as fd:
            self.assertEqual(fd.read(), "")

    def test_write(self):
        self.filename=mkstemp()[1]
        content="""q2\naze23\n"""
        self.run_test("""def _write(filename):\n f=open(filename,'a+')\n n = f.write("""+str('str("""q2\naze23\n""")')+""")\n f.close()\n return n""", self.filename, _write=[str])

        with open(self.filename) as fd:
            self.assertEqual(fd.read(), content * 2)

    def test_writelines(self):
        self.filename=mkstemp()[1]
        content=["azerty", "qsdfgh", "12345524"]
        self.run_test("""
            def _writelines(filename,_content):
                f = open(filename,'a+')
                f.writelines(_content)
                f.close()""",
            self.filename, content, _writelines=[str, List[str]])

        with open(self.filename) as fd:
            self.assertEqual(fd.read(), str().join(content)*2)

    def test_close(self):
        filename=mkstemp()[1]
        self.run_test("""
            def file_close(filename):
                f = open(filename,'w')
                f.close()
                try:
                    f.write("q")
                except:
                    pass""",
            filename, file_close=[str])

    def test_truncate(self):
        self.tempfile()
        self.run_test("""
            def _truncate(filename):
                f = open(filename)
                f.truncate(3)
                c = f.read()
                f.close()
                return c""",
            self.filename, _truncate=[str])

    def test_read(self):
        self.tempfile()
        self.run_test("""
            def _read(filename):
                f = open(filename)
                c = f.read()
                f.close()
                return c""", self.filename, _read=[str])

    def test_read_size(self):
        self.tempfile()
        self.run_test("""
            def _read_size(filename, size):
                f = open(filename)
                c = f.read(size)
                f.close()
                return c""", self.filename, 10, _read_size=[str, int])

    def test_read_oversize(self):
        self.tempfile()
        self.run_test("""
            def _read_oversize(filename, size):
                f = open(filename)
                c =  f.read(size)
                f.close()
                return c""", self.filename, len(self.file_content)+5, _read_oversize=[str, int])

    def test_readline(self):
        self.tempfile()
        self.run_test("""
            def _readline(filename):
                f = open(filename)
                c = [f.readline(),f.readline(), f.readline(),f.readline(),f.readline()]
                f.close()
                return c""", self.filename, _readline=[str])

    def test_readline_size(self):
        self.tempfile()
        self.run_test("""
            def _readline_size(filename):
                f = open(filename)
                c = [f.readline(7),f.readline(3),f.readline(4),f.readline(),f.readline(10)]
                f.close()
                return c""", self.filename, _readline_size=[str])

    def test_readline_size_bis(self):
        self.tempfile()
        self.run_test("""
            def _readline_size_bis(filename):
                f = open(filename)
                c = [f.readline(4),f.readline(3),f.readline(10),f.readline(),f.readline(5)]
                f.close()
                return c""", self.filename, _readline_size_bis=[str])

    def test_readlines(self):
        self.tempfile()
        self.run_test("""
            def _readlines(filename):
                f = open(filename)
                c = f.readlines()
                f.close()
                return c""", self.filename, _readlines=[str])

    def test_offset_read(self):
        self.tempfile()
        self.run_test("""
            def _offset_read(filename):
                f = open(filename)
                f.seek(5)
                c = f.read()
                f.close()
                return c""", self.filename, _offset_read=[str])

    def test_offset_write(self):
        self.tempfile()
        self.run_test("""
            def _offset_write(filename):
                f = open(filename, "a")
                f.seek(5)
                f.write("aze")
                f.close()
                f = open(filename,"r")
                c = f.read()
                f.close()
                return c""", self.filename, prelude = self.reinit_file, _offset_write=[str])

    def test_next(self):
        self.tempfile()
        self.run_test("""
            def _next(filename):
                f = open(filename)
                c = [next(f), next(f)]
                f.close()
                return c
                """, self.filename, _next=[str])

    def test_iter(self):
        self.tempfile()
        self.run_test("""
            def _iter(filename):
                f = open(filename)
                c = [l for l in f]
                f.close()
                return c""", self.filename, _iter=[str])

    def test_fileno(self):
        self.tempfile()
        # Useless to check if same fileno, just checking if fct can be called
        self.run_test("""
            def _fileno(filename):
                f = open(filename)
                a = f.fileno()
                c = a!= 0
                f.close()
                return c""", self.filename, _fileno=[str])

    def test_isatty(self):
        self.tempfile()
        self.run_test("""
            def _isatty(filename):
                f = open(filename)
                c = f.isatty()
                f.close()
                return c""", self.filename, _isatty=[str])

    def test_truncate(self):
        self.tempfile()
        self.run_test("""
            def _truncate(filename):
                f = open(filename, 'a')
                f.seek(3)
                f.truncate()
                f.close()
                f = open(filename)
                c = f.read()
                f.close()
                return c""", self.filename, _truncate=[str])

    def test_truncate_size(self):
        self.tempfile()
        self.run_test("""
            def _truncate_size(filename):
                f = open(filename, 'a')
                f.truncate(4)
                f.close()
                f = open(filename)
                c = f.read()
                f.close()
                return c""", self.filename, _truncate_size=[str])

    def test_flush(self):
        self.tempfile()
        # Don't know how to check properly, just checking fct call.
        self.run_test("""
            def _flush(filename):
                f = open(filename, 'a')
                f.flush()
                f.close()""", self.filename, _flush=[str])

    def test_tell(self):
        self.tempfile()
        self.run_test("""
            def _tell(filename):
                f = open(filename)
                f.read(3)
                c = f.tell()
                f.close()
                return c""", self.filename, _tell=[str])

    def test_seek(self):
        self.tempfile()
        self.run_test("""
            def _seek(filename):
                f = open(filename, 'a')
                f.seek(3)
                c = f.tell()
                f.close()
                return c""", self.filename, _seek=[str])

    def test_attribute_closed(self):
        self.tempfile()
        self.run_test("""
            def _attribute_closed(filename):
                f = open(filename, 'a')
                c = f.closed
                f.close()
                return c""", self.filename, _attribute_closed=[str])

    def test_attribute_name(self):
        self.tempfile()
        self.run_test("""
            def _attribute_name(filename):
                f = open(filename, 'a')
                c = f.name
                f.close()
                return c""", self.filename, _attribute_name=[str])

    def test_attribute_mode(self):
        self.tempfile()
        self.run_test("""
            def _attribute_mode(filename):
                f = open(filename, 'a')
                c = f.mode
                f.close()
                return c""", self.filename, _attribute_mode=[str])

    def test_attribute_newlines(self):
        self.tempfile()
        self.run_test("""
            def _attribute_newlines(filename):
                f = open(filename, 'a')
                c = f.newlines
                f.close()
                return c""", self.filename, _attribute_newlines=[str])

    def test_map_iter(self):
        self.tempfile()
        self.run_test("""
            def _map_iter(filename):
                f = open(filename)
                c = list(map(lambda s: len(s), f))
                f.close()
                return c""", self.filename, _map_iter=[str])

    # The following tests insures the PROXY compatibility with rvalues
    # Note that it doesn't close the file which is not a good idea...
    def test_rvalue_write(self):
        self.filename=mkstemp()[1]
        self.run_test("""def _rvalue_write(filename):\n open(filename,'a+').write("aze")""", self.filename, _rvalue_write=[str])

    def test_rvalue_writelines(self):
        self.filename=mkstemp()[1]
        self.run_test("""def _rvalue_writelines(filename):\n open(filename,'a+').writelines(["azerty", "qsdfgh", "12345524"])""", self.filename, _rvalue_writelines=[str])

    def test_rvalue_close(self):
        filename=mkstemp()[1]
        self.run_test("""
            def _rvalue_close(filename):
                open(filename,'w').close()""", filename, _rvalue_close=[str])

    def test_rvalue_read(self):
        self.tempfile()
        self.run_test("def _rvalue_read(filename):\n return open(filename).read()", self.filename, _rvalue_read=[str])

    def test_rvalue_readline(self):
        self.tempfile()
        self.run_test("def _rvalue_readline(filename):\n return open(filename).readline()", self.filename, _rvalue_readline=[str])

    def test_rvalue_readlines(self):
        self.tempfile()
        self.run_test("def _rvalue_readlines(filename):\n return open(filename).readlines()", self.filename, _rvalue_readlines=[str])

    def test_rvalue_next(self):
        self.tempfile()
        self.run_test("""def _rvalue_next(filename):\n return next(open(filename))""", self.filename, _rvalue_next=[str])

    def test_rvalue_fileno(self):
        self.tempfile()
        # Useless to check if same fileno, just checking if fct can be called
        self.run_test("""def _rvalue_fileno(filename):\n open(filename).fileno()""", self.filename, _rvalue_fileno=[str])

    def test_rvalue_isatty(self):
        self.tempfile()
        self.run_test("""def _rvalue_isatty(filename):\n return open(filename).isatty()""", self.filename, _rvalue_isatty=[str])

    def test_rvalue_truncate(self):
        self.tempfile()
        self.run_test("""def _rvalue_truncate(filename):\n open(filename, 'a').truncate(3)""", self.filename, _rvalue_truncate=[str])

    def test_rvalue_flush(self):
        self.tempfile()
        self.run_test("""def _rvalue_flush(filename):\n open(filename, 'a').flush()""", self.filename, _rvalue_flush=[str])

    def test_rvalue_tell(self):
        self.tempfile()
        self.run_test("""def _rvalue_tell(filename):\n return open(filename, 'a').tell()""", self.filename, _rvalue_tell=[str])

    def test_rvalue_seek(self):
        self.tempfile()
        self.run_test("""def _rvalue_seek(filename):\n open(filename, 'a').seek(3)""", self.filename, _rvalue_seek=[str])



"""Module to check coding style."""
import glob
import os
import subprocess
import unittest


class FormatTest(unittest.TestCase):
    excluded_files = ['pdqsort.hpp']
    clang_format = 'clang-format-20'

    def setUp(self):
        try:
            subprocess.check_call([FormatTest.clang_format, '--version'])
        except Exception:
            self.skipTest(FormatTest.clang_format + " not available")


    def test_clang_format(self):
        """Check coding style for cpp files."""
        pythran_dir = os.path.abspath(os.path.dirname(os.path.dirname(__file__)))
        for root, _, _ in os.walk(os.path.join(pythran_dir, "pythonic")):
            for file_ in glob.glob(os.path.join(root, "*.hpp")):
                if os.path.basename(file_) in FormatTest.excluded_files:
                    continue
                cmd = "{cmd} {file} | diff -u {file} -".format(
                    cmd=FormatTest.clang_format,
                    file=os.path.join(root, file_))
                process = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE, universal_newlines=True)
                assert not process.wait(), process.stdout.read()



import unittest
from pythran.tests import TestFromDir
import os
import glob

class TestICC(TestFromDir):
    path = os.path.join(os.path.dirname(__file__),"icc")
    files = glob.glob(os.path.join(path,"*.py"))

TestICC.populate(TestICC)

if __name__ == '__main__':
    unittest.main()



from subprocess import check_call, CalledProcessError
import os
import re
import shutil
import sys
import sysconfig
import unittest

cwd = os.path.dirname(__file__)

def _get_implementation():
    if sys.implementation.name == 'pypy':
        return 'pypy'
    return 'python'

if sys.version_info > (3, 9):

    python_version = "{}{}.{}".format(_get_implementation(), sys.version_info.major,
                                          sys.version_info.minor)

    python = sys.executable

    def find_so(name, path):
        for root, dirs, files in os.walk(path):
            for filename in files:
                if re.match(name, filename):
                    return os.path.join(root, filename)

    class TestDistutils(unittest.TestCase):

        def test_setup_build(self):
            check_call([python, 'setup.py', 'build'],
                       cwd=os.path.join(cwd, 'test_distutils'))

            extra_args = ['--no-build-isolation',]
            check_call([python, '-m', 'pip', 'install', '.',
                        '--prefix=demo_install'] + extra_args,
                       cwd=os.path.join(cwd, 'test_distutils'))

            base = os.path.join(cwd, 'test_distutils', 'demo_install',)
            libdir = os.path.join(base, 'lib')
            if not os.path.isdir(libdir):
                libdir = os.path.join(base, 'lib64')

            local_env = os.environ.copy()
            local_env['PYTHONPATH'] = os.path.join(libdir, python_version, 'site-packages')
            check_call([python, '-c', 'import demo'], env=local_env)
            check_call([python, 'setup.py', 'clean'],
                       cwd=os.path.join(cwd, 'test_distutils'))
            shutil.rmtree(os.path.join(cwd, 'test_distutils', 'demo_install'))
            shutil.rmtree(os.path.join(cwd, 'test_distutils', 'build'))

        def test_invalid_compiler(self):
            try:
                cxx = os.environ.get('CXX')
                os.environ['CXX'] = "ankou"
                self.test_setup_build()
                self.assertFalse(true)
            except CalledProcessError:
                pass
            finally:
                del os.environ['CXX']
                if cxx is not None:
                    os.environ['CXX'] = cxx


        def test_setup_sdist_install(self):
            check_call([python, 'setup.py', 'sdist', "--dist-dir=sdist"],
                       cwd=os.path.join(cwd, 'test_distutils'))
            check_call(['tar', 'xzf', 'demo-1.0.tar.gz'],
                       cwd=os.path.join(cwd, 'test_distutils', 'sdist'))
            check_call([python, 'setup.py', 'install', '--prefix=demo_install'],
                       cwd=os.path.join(cwd, 'test_distutils', 'sdist', 'demo-1.0'))
            shutil.rmtree(os.path.join(cwd, 'test_distutils', 'sdist'))

        def test_setup_bdist_install(self):
            check_call([python, 'setup.py', 'bdist', "--dist-dir=bdist"],
                       cwd=os.path.join(cwd, 'test_distutils'))
            dist_path = os.path.join(cwd, 'test_distutils', 'bdist')
            tgz = [f for f in os.listdir(dist_path) if f.endswith(".tar.gz")][0]
            check_call(['tar', 'xzf', tgz], cwd=dist_path)

            demo_so = find_so(r"demo.*\.so", dist_path)
            self.assertIsNotNone(demo_so)
            shutil.rmtree(dist_path)

        def test_setup_wheel_install(self):
            check_call([python, 'setup.py', 'bdist_wheel', "--dist-dir=bdist_wheel"],
                       cwd=os.path.join(cwd, 'test_distutils_setuptools'))
            dist_path = os.path.join(cwd, 'test_distutils_setuptools', 'bdist_wheel')
            wheel_dir = 'wheeeeeeel'
            whl = [f for f in os.listdir(dist_path) if f.endswith(".whl")][0]
            check_call(['unzip', whl, '-d', wheel_dir], cwd=dist_path)

            demo_so = find_so(r"demo.*\.so", os.path.join(dist_path, wheel_dir))
            self.assertIsNotNone(demo_so)
            shutil.rmtree(dist_path)


        def test_setup_build2(self):
            check_call([python, 'setup.py', 'build'],
                       cwd=os.path.join(cwd, 'test_distutils_packaged'))

            extra_args = ['--no-build-isolation',]
            check_call([python, '-m', 'pip', 'install', '.',
                        '--prefix=demo_install2'] + extra_args,
                       cwd=os.path.join(cwd, 'test_distutils_packaged'))

            base = os.path.join(cwd, 'test_distutils_packaged', 'demo_install2',)
            libdir = os.path.join(base, 'lib')
            if not os.path.isdir(libdir):
                libdir = os.path.join(base, 'lib64')
            local_env = os.environ.copy()
            local_env['PYTHONPATH'] = os.path.join(libdir, python_version, 'site-packages')
            check_call([python, '-c', 'import demo2.a'], env=local_env)
            check_call([python, 'setup.py', 'clean'],
                       cwd=os.path.join(cwd, 'test_distutils_packaged'))
            shutil.rmtree(os.path.join(cwd, 'test_distutils_packaged', 'demo_install2'))
            shutil.rmtree(os.path.join(cwd, 'test_distutils_packaged', 'build'))

        def test_setup_sdist_install2(self):
            check_call([python, 'setup.py', 'sdist', "--dist-dir=sdist2"],
                       cwd=os.path.join(cwd, 'test_distutils_packaged'))
            check_call(['tar', 'xzf', 'demo2-1.0.tar.gz'],
                       cwd=os.path.join(cwd, 'test_distutils_packaged', 'sdist2'))
            check_call([python, 'setup.py', 'install', '--prefix=demo_install2'],
                       cwd=os.path.join(cwd, 'test_distutils_packaged', 'sdist2', 'demo2-1.0'))
            shutil.rmtree(os.path.join(cwd, 'test_distutils_packaged', 'sdist2'))

        def test_setup_bdist_install2(self):
            check_call([python, 'setup.py', 'bdist', "--dist-dir=bdist"],
                       cwd=os.path.join(cwd, 'test_distutils_packaged'))
            dist_path = os.path.join(cwd, 'test_distutils_packaged', 'bdist')
            tgz = [f for f in os.listdir(dist_path) if f.endswith(".tar.gz")][0]
            check_call(['tar', 'xzf', tgz], cwd=dist_path)

            demo_so = find_so(r"a.*\.so", dist_path)
            self.assertIsNotNone(demo_so)
            shutil.rmtree(dist_path)

        @unittest.skipIf(sys.version_info >= (3, 12), "setup install is deprecated")
        def test_setup_build3(self):
            check_call([python, 'setup.py', 'build'],
                       cwd=os.path.join(cwd, 'test_distutils_numpy'))

            base = os.path.join(cwd, 'test_distutils_numpy', 'demo_install3',)
            libdir = os.path.join(base, 'lib')
            lib64dir = os.path.join(base, 'lib64')

            local_env = os.environ.copy()
            local_env['PYTHONPATH'] = os.pathsep.join((
                os.path.join(lib64dir, python_version,
                             'site-packages'),
                os.path.join(libdir, python_version,
                             'site-packages'),
                local_env.get('PYTHONPATH', ''),
                ))

            check_call([python, 'setup.py', 'install', '--prefix=demo_install3'],
                       cwd=os.path.join(cwd, 'test_distutils_numpy'),
                       env=local_env)

            check_call([python, '-c', 'import demo3.a'],
                       cwd=cwd, env=local_env)
            check_call([python, 'setup.py', 'clean'],
                       cwd=os.path.join(cwd, 'test_distutils_numpy'))
            shutil.rmtree(os.path.join(cwd, 'test_distutils_numpy', 'demo_install3'))
            shutil.rmtree(os.path.join(cwd, 'test_distutils_numpy', 'build'))

        def test_setup_sdist_install3(self):
            check_call([python, 'setup.py', 'sdist', "--dist-dir=sdist3"],
                       cwd=os.path.join(cwd, 'test_distutils_numpy'))
            check_call(['tar', 'xzf', 'demo3-1.0.tar.gz'],
                       cwd=os.path.join(cwd, 'test_distutils_numpy', 'sdist3'))
            check_call([python, 'setup.py', 'install', '--prefix=demo_install3'],
                       cwd=os.path.join(cwd, 'test_distutils_numpy', 'sdist3', 'demo3-1.0'))
            shutil.rmtree(os.path.join(cwd, 'test_distutils_numpy', 'sdist3'))

        def test_setup_bdist_install3(self):
            check_call([python, 'setup.py', 'bdist', "--dist-dir=bdist"],
                       cwd=os.path.join(cwd, 'test_distutils_numpy'))
            dist_path = os.path.join(cwd, 'test_distutils_numpy', 'bdist')
            tgz = [f for f in os.listdir(dist_path) if f.endswith(".tar.gz")][0]
            check_call(['tar', 'xzf', tgz], cwd=dist_path)

            demo_so = find_so(r"a.*\.so", dist_path)
            self.assertIsNotNone(demo_so)
            shutil.rmtree(dist_path)

    try:
        check_call(['ninja', '--version'])
        has_ninja = True
    except:
        has_ninja = False

    try:
        check_call(['meson', '--version'])
        has_meson = True
    except:
        has_meson = False

    class TestMeson(unittest.TestCase):

        @unittest.skipIf(not has_meson, "meson not found")
        @unittest.skipIf(not has_ninja, "ninja not found")
        def test_meson_build(self):
            srcdir = os.path.join(cwd, 'test_distutils')
            builddir = os.path.join(srcdir, '_meson_build')
            shutil.rmtree(builddir, ignore_errors=True)
            check_call(['meson', 'setup', builddir, '.'],
                       cwd=srcdir)
            check_call(['ninja', '-v'],
                       cwd=builddir)
            check_call([python, '-c', 'import b'],
                       cwd=builddir)




import numpy as np

#pythran export max_len_seq_inner(int64 [], int8[], int, int, int8[])
#runas import numpy as np; x = np.arange(10, dtype=np.int64); y = np.arange(10, dtype=np.int8); z = np.arange(10, dtype=np.int8); max_len_seq_inner(x, y, 10, 2, z)

def max_len_seq_inner(taps, state, nbits, length, seq):
    # Here we compute MLS using a shift register, indexed using a ring buffer
    # technique (faster than using something like np.roll to shift)
    n_taps = taps.shape[0]
    idx = 0
    for i in range(length):
        feedback = state[idx]
        seq[i] = feedback
        for tap in taps:
            feedback ^= state[(tap + idx) % nbits]
        state[idx] = feedback
        idx = (idx + 1) % nbits
    # state must be rolled s.t. next run, when idx==0, it's in the right place
    return np.roll(state, -idx, axis=0)



"""Downsample data using the Largest-Triangle-Three-Buckets algorithm.

Translated for Pythran from https://git.sr.ht/~javiljoen/lttb-numpy
which is Copyright (c) 2020, JA Viljoen, MIT license.

Reference
---------
Sveinn Steinarsson. 2013. Downsampling Time Series for Visual
Representation. MSc thesis. University of Iceland.
"""

#runas import numpy as np; tseries = np.arange(200.).reshape(100, 2); downsample(tseries, 8)

import numpy as np

def _areas_of_triangles(a, bs, c):
    bs_minus_a = bs - a
    a_minus_bs = a - bs
    return 0.5 * abs(
        (a[0] - c[0]) * (bs_minus_a[:, 1]) - (a_minus_bs[:, 0]) * (c[1] - a[1])
    )


#pythran export downsample(float[:, :] order(C), int)
def downsample(data, n_out):
    if n_out > data.shape[0]:
        raise ValueError("n_out must be <= number of rows in data")

    if n_out == data.shape[0]:
        return data

    if n_out < 3:
        raise ValueError("Can only downsample to a minimum of 3 points")

    # Split data into bins
    n_bins = n_out - 2
    data_bins = np.array_split(data[1 : len(data) - 1], n_bins)

    # Prepare output array
    # First and last points are the same as in the input.
    out = np.zeros((n_out, 2))
    out[0] = data[0]
    out[len(out) - 1] = data[len(data) - 1]

    # Largest Triangle Three Buckets (LTTB):
    # In each bin, find the point that makes the largest triangle
    # with the point saved in the previous bin
    # and the centroid of the points in the next bin.
    for i in range(len(data_bins)):
        this_bin = data_bins[i]

        if i < n_bins - 1:
            next_bin = data_bins[i + 1]
        else:
            next_bin = data[len(data) - 1 :]

        a = out[i]
        bs = this_bin
        c = next_bin.mean(axis=0)

        areas = _areas_of_triangles(a, bs, c)
        out[i + 1] = bs[np.argmax(areas)]

    return out



from pythran.tests import TestEnv

import unittest
from tempfile import mkstemp
import os
import sys


class TestArray(TestEnv):

    @unittest.skipIf(sys.implementation.name == "pypy", "non supported upstream")
    def test_typecodes(self):
        self.run_test(
            "def typecodes_(i): import array; return array.typecodes[i]",
            2,
            typecodes_=[int],
        )

    def test_array_empty(self):
        self.run_test(
            "def array_empty_(): import array; return len(array.array('b'))",
            array_empty_=[],
        )

    def test_array_seq(self):
        self.run_test(
            "def array_seq_(): import array; return len(array.array('h', [1, 2, 3]))",
            array_seq_=[],
        )

    def test_array_seq_tuple(self):
        self.run_test(
            "def array_seq_tuple_(): import array; return len(array.array('h', (8, 9, 10)))",
            array_seq_tuple_=[],
        )

    def test_array_tolist(self):
        self.run_test(
            "def array_tolist_(n): import array; return array.array('h', [n]).tolist()",
            2,
            array_tolist_=[int],
        )

    def test_array_append(self):
        self.run_test(
            "def array_append_(n): import array; x = array.array('h', [n]); x.append(1); return x.tolist()",
            2,
            array_append_=[int],
        )

    def test_array_buffer_info(self):
        self.run_test(
            "def array_buffer_info_(n): import array; x = array.array('h', [n]); return x.buffer_info()[1]",
            2,
            array_buffer_info_=[int],
        )

    def test_array_byteswap(self):
        self.run_test(
            "def array_byteswap_(n): import array; x = array.array('H', [n]); x.byteswap(); return x.tolist()",
            2,
            array_byteswap_=[int],
        )

    def test_array_count(self):
        self.run_test(
            "def array_count_(n): import array; x = array.array('H', [n, n]); return x.count(1), x.count(2)",
            2,
            array_count_=[int],
        )

    def test_array_extend(self):
        self.run_test(
            "def array_extend_(n): import array; x = array.array('h', [n]); x.extend((1, 1)); return x.tolist()",
            2,
            array_extend_=[int],
        )

    def test_array_fromfile(self):
        filename = mkstemp()[1]
        with open(filename, "w") as fd:
            fd.write("12345678" * 100)
        self.run_test(
            "def array_fromfile_(s): import array; x = array.array('h'); x.fromfile(open(s, 'rb'), 8); return x.tolist()",
            filename,
            array_fromfile_=[str],
        )
        os.remove(filename)

    def test_array_fromlist(self):
        self.run_test(
            "def array_fromlist_(f): import array; x = array.array('f'); x.fromlist([f]); return x.tolist()",
            3.0,
            array_fromlist_=[float],
        )

    def test_array_frombytes(self):
        filename = mkstemp()[1]
        with open(filename, "w") as fd:
            fd.write("12345678" * 10)
        self.run_test(
            "def array_frombytes_(s): import array; x = array.array('i'); x.frombytes(open(s, 'rb').read()); return x.tolist()",
            filename,
            array_frombytes_=[str],
        )
        os.remove(filename)

    def test_array_index(self):
        self.run_test(
            "def array_index_(f): import array; x = array.array('I',[1,2,3]); return x.index(f)",
            3,
            array_index_=[int],
        )

    def test_array_insert(self):
        self.run_test(
            "def array_insert_(f): import array; x = array.array('I',[1,2,3]); x.insert(0, f); x.insert(-1, f); return x.tolist()",
            3,
            array_insert_=[int],
        )

    def test_array_pop(self):
        self.run_test(
            "def array_pop_(f): import array; x = array.array('I',[f,2,3]); x.pop(); return int(x.pop(0))",
            3,
            array_pop_=[int],
        )

    def test_array_remove(self):
        self.run_test(
            "def array_remove_(f): import array; x = array.array('I',[f,2,3]); x.remove(2); return x.tolist()",
            3,
            array_remove_=[int],
        )

    def test_array_reverse(self):
        self.run_test(
            "def array_reverse_(f): import array; x = array.array('I',[f,2,3]); x.reverse(); return x.tolist()",
            3,
            array_reverse_=[int],
        )

    def test_array_get(self):
        self.run_test(
            "def array_get_(f): import array; x = array.array('I',[f,2,3]); return x[0], x[f]",
            1,
            array_get_=[int],
        )

    def test_array_set(self):
        self.run_test(
            "def array_set_(f): import array; x = array.array('I',[f,2,3]); x[2] = 4; x[f] = 9; return x.tolist()",
            1,
            array_set_=[int],
        )

    def test_array_iter(self):
        self.run_test(
            "def array_iter_(f): import array; x = array.array('I',[f,2,3]); return next(iter(x))",
            1,
            array_iter_=[int],
        )

    def test_array_len(self):
        self.run_test(
            "def array_len_(f): import array; x = array.array('I',[f,2,3]); return len(x)",
            1,
            array_len_=[int],
        )

    def test_array_loop(self):
        self.run_test(
            "def array_loop_(f): import array; x = "
            "array.array('I',[f,2,3]); return [2 * i for i in x]",
            1,
            array_loop_=[int],
        )

    def test_array_add(self):
        self.run_test(
            "def array_add_(f): import array; x = "
            "array.array('I',[f,2,3]); return (x + x).tolist()",
            1,
            array_add_=[int],
        )

    def test_array_slice(self):
        self.run_test(
            "def array_slice_(f): import array; x = array.array('I',[f,2,3]); "
            "return x[1:].tolist(), x[:-1][0]",
            1,
            array_slice_=[int],
        )
